local idTweaks = LibStub('AceAddon-3.0'):GetAddon('idTweaks')
local MailModule = idTweaks:NewModule('Mail', 'AceConsole-3.0', 'AceEvent-3.0')

function MailModule:setDefaults()
  --MailModule:Print('MailModule:setDefaults')

  self.defaults = {
    profile = {
      autoSendMail = true
    }
  }
end

function MailModule:setDatabase()
  --MailModule:Print('MailModule:setDatabase')

  self.db = idTweaks.db:RegisterNamespace('Mail', self.defaults)
end

function MailModule:setOptions()
  --MailModule:Print('MailModule:setOptions')

  self.options = {
    name = 'Mail',
    handler = self,
    type = 'group',
    args = {
      autoSendMail = {
        order = 1,
        name = 'Auto Send',
        desc = 'When the mail has a Recipient and all item slots are filled, send it.',
        handler = self,
        type = 'toggle',
        get = function()
          return self.db.profile.autoSendMail
        end,
        set = function(_, value)
          self.db.profile.autoSendMail = value
        end
      }
    },
  }
end

function MailModule:registerOptions()
  --MailModule:Print('MailModule:registerOptions')

  LibStub('AceConfig-3.0'):RegisterOptionsTable('Mail', self.options)
  LibStub('AceConfigDialog-3.0'):AddToBlizOptions('Mail', 'Mail', 'idTweaks')
end

function MailModule:OnInitialize()
  --MailModule:Print('MailModule:OnInitialize')

  self:setDefaults()
  self:setDatabase()
  self:setOptions()
end

function MailModule:OnEnable()
  --MailModule:Print('MailModule:OnEnable')

  self:registerOptions()

  -- Register for events
  self:RegisterEvent('MAIL_SEND_INFO_UPDATE')
end

function MailModule:OnDisable()
  --MailModule:Print('MailModule:OnDisable')
end

-- Gets the number of slots that are free in a mail.
function MailModule:getFreeMailSlots()
  --MailModule:Print('MailModule:getFreeMailSlots')

  local freeSlots = 0

  for slot = 1, 12 do
    itemName, _, _, _ = GetSendMailItem(slot)

    if not itemName then
      freeSlots = freeSlots + 1
    end
  end

  return freeSlots
end

-- Returns true if the mail has all items attached.
function MailModule:mailIsFull()
  --MailModule:Print('MailModule:mailIsFull')

  return self:getFreeMailSlots() == 0
end

-- Gets the recipient from the mail editbox.
function MailModule:getMailRecipient()
  --MailModule:Print('MailModule:getMailRecipient')

  return SendMailNameEditBox:GetText()
end

-- Returns true if the mail has a recipient.
function MailModule:mailHasRecipient()
  --MailModule:Print('MailModule:mailHasRecipient')

  return self:getMailRecipient() ~= ''
end

-- Gets the subject from the mail editbox.
function MailModule:getMailSubject()
  --MailModule:Print('MailModule:getMailSubject')

  return SendMailSubjectEditBox:GetText()
end

-- Returns true if the mail has a recipient.
function MailModule:mailHasSubject()
  --MailModule:Print('MailModule:mailHasSubject')

  return self:getMailSubject() ~= ''
end

-- Gets the body from the mail editbox.
function MailModule:getMailBody()
  --MailModule:Print('MailModule:getMailBody')

  return SendMailScrollChildFrame:GetText()
end

-- Returns true if the mail can be sent.
function MailModule:canSendMail()
  --MailModule:Print('MailModule:canSendMail')

  return self:mailHasRecipient() and self:mailHasSubject() and self:mailIsFull()
end

-- Sends the mail.
function MailModule:sendMail()
  --MailModule:Print('MailModule:sendMail')

  local recipient = self:getMailRecipient()
  local subject = self:getMailSubject()
  -- local body = self:getMailBody()

  SendMail(recipient, subject, '')
end

function MailModule:MAIL_SEND_INFO_UPDATE()
  --MailModule:Print('MailModule:MAIL_SEND_INFO_UPDATE')

  if self.db.profile.autoSendMail == false then
    return
  end

  if self:canSendMail() then
    self:sendMail()
  end
end

idTweaks.Mail = MailModule
